library(ShortRead);
library(MASS);

getReadCnts = function(dir, infile, outfile, format="fastq") {
  fq = readFastq(dir, infile);
  reads = sread(fq);
  tbls = tables(reads, n=length(unique(reads)));
  write.table(tbls$top, file=outfile, quote=F, col.names=F, row.names=T);
  return(tbls$top);
}

getShadow2Cnts.new = function (tag.idx, tags, counts, method="all") {
  tag.chunk = tags[tag.idx];

  if ( method == "all" ) {
    sub.neigh.mx = getSub2ShadowCnts (tag.idx, tags);
    ins.neigh.mx = getIns2ShadowCnts (tag.idx, tags);
    del.neigh.mx = getDel2ShadowCnts(tag.idx, tags);
    neigh.mx = rbind(sub.neigh.mx, ins.neigh.mx, del.neigh.mx);
  }
  if ( method == "sub" ) {
    neigh.mx = getSub2ShadowCnts (tag.idx, tags);
  }
  if ( method == "indel" ) {
    ins.neigh.mx = getIns2ShadowCnts (tag.idx, tags);
    del.neigh.mx = getDel2ShadowCnts(tag.idx, tags);
    neigh.mx = rbind(ins.neigh.mx, del.neigh.mx);
  }

  ### this doesn't work for indels right now ###
  ### because indels don't have position info for now, and not implemented 2
  ### changes
  ### neigh.mx[,1] is tag, neigh.mx[,3] is neighbor
  ### neigh.mx[,2] is position of 2nd sub, neigh.mx[,4] is position of 1st sub
#  neigh.mx.in = neigh.mx[which(is.element(neigh.mx[,3], tags)),]  
  which.in = which(is.element(neigh.mx[,3], tags[1001:length(tags)]))
  neigh.mx.in = neigh.mx[which.in,];
  if ( length(which.in) == 1 ) {
    neigh.mx.in = t(data.frame(neigh.mx.in));
  }
  idx.in = match(neigh.mx.in[,3], tags);

  counts.in = counts[idx.in]
  tags.in = tags[idx.in];
  names(counts.in) = tags.in;
  counts.in.adj = counts.in / table(tags.in)[names(counts.in)]  
  counts.in = counts.in.adj;

  shadow.cnts = tapply(counts.in, neigh.mx.in[,1], sum)
  tag.cnts = counts[tag.idx[match(names(shadow.cnts), tag.chunk)]]

  shadow.table = tapply(counts.in, neigh.mx.in[,1], table)

  # by position doesn't work for neigh2's -- should get joint distn
  shadow.cnts.base = tapply(counts.in,
    list(neigh.mx.in[,1], neigh.mx.in[,2]), sum)

  shadow.cnts.base2 = tapply(counts.in,
    list(neigh.mx.in[,1], neigh.mx.in[,4]), sum)

  shadow.cnts.base[is.na(shadow.cnts.base)] = 0;
  shadow.cnts.base2[is.na(shadow.cnts.base2)] = 0;

  read.length = nchar(tags[1]);

  missing.col = setdiff(1:(read.length-1), as.numeric(colnames(shadow.cnts.base)))
  missing.col2 = setdiff(2:read.length, as.numeric(colnames(shadow.cnts.base2)))

  if ( length(missing.col) > 0 ) {
    add.col = matrix(0, nrow(shadow.cnts.base), length(missing.col))
    colnames(add.col) = as.character(missing.col);
    shadow.cnts.base = cbind(shadow.cnts.base, add.col)
  }
  if ( length(missing.col2) > 0 ) {
    add.col = matrix(0, nrow(shadow.cnts.base), length(missing.col2))
    colnames(add.col) = as.character(missing.col2);
    shadow.cnts.base2 = cbind(shadow.cnts.base2, add.col)
  }
  
  if ( nrow(shadow.cnts.base) == 1 ) {
    shadow.cnts.base =
      t(matrix(shadow.cnts.base[,order(as.numeric(colnames(shadow.cnts.base)))]))
  } else {
    shadow.cnts.base =
      shadow.cnts.base[,order(as.numeric(colnames(shadow.cnts.base)))]
  }
  colnames(shadow.cnts.base) = as.character(1:ncol(shadow.cnts.base));
  if ( nrow(shadow.cnts.base2) == 1 ) {
    shadow.cnts.base2 =
      t(matrix(shadow.cnts.base2[,order(as.numeric(colnames(shadow.cnts.base2)))]));
  } else {
    shadow.cnts.base2 =
      shadow.cnts.base2[,order(as.numeric(colnames(shadow.cnts.base2)))]
  }
  colnames(shadow.cnts.base2) = as.character(2:(ncol(shadow.cnts.base2)+1));
  common.pos = as.character(2:(read.length-1));

  if ( nrow(shadow.cnts.base) == 1 ) {
    shadow.cnts.base1and2 = t(data.frame(c(shadow.cnts.base[,"1"], 
      (shadow.cnts.base[,common.pos] + shadow.cnts.base2[,common.pos]),
      shadow.cnts.base2[,as.character(read.length)] )));
  } else {
    shadow.cnts.base1and2 = cbind(shadow.cnts.base[,"1"], 
      (shadow.cnts.base[,common.pos] + shadow.cnts.base2[,common.pos]),
      shadow.cnts.base2[,as.character(read.length)] );
  }
  colnames(shadow.cnts.base1and2) = 1:read.length;
  
  if ( length(tag.cnts) < length(tag.idx) ) {
    idx.tags.no.shadow = which(!is.element(tag.chunk, neigh.mx.in[,1]))
    tag.cnts.no.shadow = counts[tag.idx[idx.tags.no.shadow]];
    shadow.cnts.no.shadow = rep(0, length(tag.cnts.no.shadow));
    tag.cnts = c(tag.cnts, tag.cnts.no.shadow);
    shadow.cnts = c(shadow.cnts, shadow.cnts.no.shadow);
    shadow.cnts.base.no.shadow = matrix(0, length(tag.cnts.no.shadow),
      (ncol(shadow.cnts.base)+1));
    shadow.cnts.base1and2 = rbind(shadow.cnts.base1and2,
      shadow.cnts.base.no.shadow);

    shadow.table = c(shadow.table, shadow.cnts.no.shadow);
  }
  return(list(ts.cnts=cbind(tag.cnts, shadow.cnts), 
              shadow.by.base=shadow.cnts.base1and2,
	      shadow.table = shadow.table) );
}

getShadowCnts.new = function (tag.idx, tags, counts, method="all") {
  tag.chunk = tags[tag.idx];

  if ( method == "all" ) {
    sub.neigh.mx = getSubShadowCnts (tag.idx, tags);
    ins.neigh.mx = getInsShadowCnts (tag.idx, tags);
    del.neigh.mx = getDelShadowCnts(tag.idx, tags);
    neigh.mx = rbind(sub.neigh.mx, ins.neigh.mx, del.neigh.mx);
  }
  if ( method == "sub" ) {
    neigh.mx = getSubShadowCnts (tag.idx, tags);
  }
  if ( method == "indel" ) {
    ins.neigh.mx = getInsShadowCnts (tag.idx, tags);
    del.neigh.mx = getDelShadowCnts(tag.idx, tags);
    neigh.mx = rbind(ins.neigh.mx, del.neigh.mx);
  }

  ### this doesn't work for indels right now ###
  ### indels don't have position info in the matrix

  # possible neighbors that are actually present in the sample
  neigh.mx.in = neigh.mx[which(is.element(neigh.mx[,3], tags[1001:length(tags)])),]
  # which reads match these neighbors
  idx.in = match(neigh.mx.in[,3], tags);
  # remove neighbors that match top tags
  #idx.in.filterTop = idx.in[-which(idx.in<=1000)]
  counts.in = counts[idx.in];
  tags.in = tags[idx.in];
  names(counts.in) = tags.in;
  counts.in.adj = counts.in / table(tags.in)[names(counts.in)]  
  counts.in = counts.in.adj;

#  counts.in = counts[match(neigh.mx.in[,3], tags)]
  shadow.cnts = tapply(counts.in, neigh.mx.in[,1], sum)
  tag.cnts = counts[tag.idx[match(names(shadow.cnts), tag.chunk)]]

  shadow.table = tapply(counts.in, neigh.mx.in[,1], table)

  # neigh.mx.in[,1] is tag
  # neigh.mx.in[,2] is base position
  shadow.cnts.base = tapply(counts.in,
    list(neigh.mx.in[,1], neigh.mx.in[,2]), sum)

  shadow.cnts.base[is.na(shadow.cnts.base)] = 0;
  shadow.cnts.base =
    shadow.cnts.base[,order(as.numeric(colnames(shadow.cnts.base)))]

  if ( ncol(shadow.cnts.base) < nchar(neigh.mx[1,1]) ) {
    shadow.cnts.base.full = matrix(0, nrow(shadow.cnts.base), nchar(neigh.mx[1,1]));
    colnames(shadow.cnts.base.full) = 1:ncol(shadow.cnts.base.full);
    shadow.cnts.base.full[, colnames(shadow.cnts.base)] = shadow.cnts.base;
    shadow.cnts.base = shadow.cnts.base.full;
  }

  if ( length(tag.cnts) < length(tag.idx) ) {
    idx.tags.no.shadow = which(!is.element(tag.chunk, neigh.mx.in[,1]))
    tag.cnts.no.shadow = counts[tag.idx[idx.tags.no.shadow]];
    shadow.cnts.no.shadow = rep(0, length(tag.cnts.no.shadow));
    tag.cnts = c(tag.cnts, tag.cnts.no.shadow);
    shadow.cnts = c(shadow.cnts, shadow.cnts.no.shadow);
#    shadow.cnts.base.no.shadow = matrix(0, length(tag.cnts.no.shadow), (ncol(shadow.cnts.base)));
    shadow.cnts.base.no.shadow = matrix(0, length(tag.cnts.no.shadow), (nchar(neigh.mx[1,1])));
    if (nrow(shadow.cnts.base)>0) {
      shadow.cnts.base = rbind(shadow.cnts.base, shadow.cnts.base.no.shadow);
    } else {
      shadow.cnts.base = shadow.cnts.base.no.shadow;
    }
    shadow.table = c(shadow.table, shadow.cnts.no.shadow);
  }
  return(list(ts.cnts=cbind(tag.cnts, shadow.cnts), 
              shadow.by.base=shadow.cnts.base,
	      shadow.table=shadow.table ) );
}

getSubShadowCnts = function (tag.idx, tags) {
  tag.chunk = tags[tag.idx];

  subNeighs = lapply(tag.chunk, getSubNeighbors);
  sub.nrow = sum(unlist(lapply(subNeighs, nrow)))
  sub.neigh.mx = matrix(nrow=sub.nrow, ncol=3);

  for ( i in 1:length(tag.chunk) ) {
    if ( i==1 ) {
      idx = 1:nrow(subNeighs[[i]]);
    } else {
      idx = (idx[length(idx)]+1):(idx[length(idx)]+nrow(subNeighs[[i]]));
    }
    sub.neigh.mx[idx,] = subNeighs[[i]]
  }
  return(sub.neigh.mx);
}

getSub2ShadowCnts = function (tag.idx, tags) {
  tag.chunk = tags[tag.idx];

#  subNeighs = lapply(tag.chunk, getSub2Neighbors);
  subNeighs = lapply(tag.chunk, getSub2OnlyNeighbors);  
  sub.nrow = sum(unlist(lapply(subNeighs, nrow)))
  # ncol 3 or 4 depending on if 4th col
  sub.neigh.mx = matrix(nrow=sub.nrow, ncol=4);

  for ( i in 1:length(tag.chunk) ) {
    if ( i==1 ) {
      idx = 1:nrow(subNeighs[[i]]);
    } else {
      idx = (idx[length(idx)]+1):(idx[length(idx)]+nrow(subNeighs[[i]]));
    }
    sub.neigh.mx[idx,] = subNeighs[[i]]
  }
  return(sub.neigh.mx);
}

getSubNeighbors = function (tag) {
  tagNeighbors = unlist(lapply(1:nchar(tag), subOneBase, tag=tag));
  cbind(rep(tag, length(tagNeighbors)),
        rep(1:nchar(tag), each=3),
        tagNeighbors);
}

getSub2OnlyNeighbors = function (tag) {
  tagNeighbors = unlist(lapply(1:nchar(tag), subOneBase, tag=tag));
  neigh1.mx = cbind(rep(tag, length(tagNeighbors)),
    rep(1:nchar(tag), each=3),
    tagNeighbors);

  ### need to change the tag column back to the org tag, not the one after
  ### one base of change
  neigh2.list = lapply(tagNeighbors, getSubNeighbors)
  neigh2.mx = neigh2.list[[1]];
  for ( i in 2:length(neigh2.list) ) {
    neigh2.mx = rbind(neigh2.mx, neigh2.list[[i]]);
  }  

  neigh2.mx = cbind(neigh2.mx, rep(neigh1.mx[,2], each=nrow(neigh1.mx)))
  
  idx.tag = which(neigh2.mx[,3] == tag);
  idx.neigh1 = which(is.element(neigh2.mx[,3], tagNeighbors));
  idx.pos2lepos1 = which(as.numeric(neigh2.mx[,4]) <= as.numeric(neigh2.mx[,2]))
  idx.rm = c(idx.tag, idx.neigh1, idx.pos2lepos1);
  neigh2.mx = neigh2.mx[-idx.rm,];

  neigh.mx = neigh2.mx;
#  neigh.mx = rbind(neigh1.mx, neigh2.mx);
  neigh.mx[,1] = rep(tag, nrow(neigh.mx))
  return(neigh.mx);
}

subOneBase = function(i, tag) {
  org = tag;
  neighbors = vector(length=4);
  substr(tag, i, i) = "A";
  neighbors[1] = tag;
  substr(tag, i, i) = "C";
  neighbors[2] = tag;
  substr(tag, i, i) = "G";
  neighbors[3] = tag;
  substr(tag, i, i) = "T";    
  neighbors[4] = tag;
  neighbors = neighbors[-which(neighbors==org)]
}

getInsShadowCnts = function (tag.idx, tags) {
  tag.chunk = tags[tag.idx];
  
  insNeighs = lapply(tag.chunk, getInsNeighbors);
  ins.nrow = sum(unlist(lapply(insNeighs, nrow)))
  ins.neigh.mx = matrix(nrow=ins.nrow, ncol=3);
  for ( i in 1:length(tag.chunk) ) {
    if ( i==1 ) {
      idx = 1:nrow(insNeighs[[i]]);
    } else {
      idx = (idx[length(idx)]+1):(idx[length(idx)]+nrow(insNeighs[[i]]));
    }
    ins.neigh.mx[idx,] = insNeighs[[i]]
  }
  return(ins.neigh.mx);
}

getDelShadowCnts = function (tag.idx, tags) {
  tag.chunk = tags[tag.idx];
  
  delNeighs = lapply(tag.chunk, getDelNeighbors);
  del.nrow = sum(unlist(lapply(delNeighs, nrow)))
  del.neigh.mx = matrix(nrow=del.nrow, ncol=3);
  for ( i in 1:length(tag.chunk) ) {
    if ( i==1 ) {
      idx = 1:nrow(delNeighs[[i]]);
    } else {
      idx = (idx[length(idx)]+1):(idx[length(idx)]+nrow(delNeighs[[i]]));
    }
    del.neigh.mx[idx,] = delNeighs[[i]]
  }
  return(del.neigh.mx);
}

getIns2ShadowCnts = function (tag.idx, tags) {
  tag.chunk = tags[tag.idx];

  insNeighs = lapply(tag.chunk, getIns2OnlyNeighbors);  
  ins.nrow = sum(unlist(lapply(insNeighs, nrow)))
  # ncol 3 or 4 depending on if 4th col
  ins.neigh.mx = matrix(nrow=ins.nrow, ncol=4);

  for ( i in 1:length(tag.chunk) ) {
    if ( i==1 ) {
      idx = 1:nrow(insNeighs[[i]]);
    } else {
      idx = (idx[length(idx)]+1):(idx[length(idx)]+nrow(insNeighs[[i]]));
    }
    ins.neigh.mx[idx,] = insNeighs[[i]]
  }
  return(ins.neigh.mx);
}

getInsNeighbors = function (tag) {
  neighList = lapply(2:nchar(tag), insOneBase, tag=tag);
  positions = rep(2:nchar(tag), times=unlist(lapply(neighList, length)))
  tagNeighbors = unlist(neighList);
  cbind(rep(tag, length(tagNeighbors)), positions, tagNeighbors);
}

getIns2OnlyNeighbors = function (tag) {
  neighList = lapply(2:nchar(tag), insOneBase, tag=tag);
  positions = rep(2:nchar(tag), times=unlist(lapply(neighList, length)))
  tagNeighbors = unlist(neighList);
  neigh1.mx = cbind(rep(tag, length(tagNeighbors)), positions, tagNeighbors);

  ### need to change the tag column back to the org tag, not the one after
  ### one base of change
  neigh2.list = lapply(tagNeighbors, getInsNeighbors)
  neigh2.mx = neigh2.list[[1]];
  for ( i in 2:length(neigh2.list) ) {
    neigh2.mx = rbind(neigh2.mx, neigh2.list[[i]]);
  }  

  # add original position to col 4
  neigh2.mx = cbind(neigh2.mx, 
    rep(neigh1.mx[,2], times=unlist(lapply(neigh2.list, nrow))))
  
  # which neigh2 is the same as the orginal tag
  idx.tag = which(neigh2.mx[,3] == tag);

  # which neigh2 is the same as neigh1
  idx.neigh1 = which(is.element(neigh2.mx[,3], tagNeighbors));

  # which neigh1 pos is less than neigh2 pos
  idx.pos2lepos1 = which(as.numeric(neigh2.mx[,4]) <= as.numeric(neigh2.mx[,2]))
  idx.rm = c(idx.tag, idx.neigh1, idx.pos2lepos1);
  neigh2.mx = neigh2.mx[-idx.rm,];

  neigh.mx = neigh2.mx;
  neigh.mx[,1] = rep(tag, nrow(neigh.mx))
  return(neigh.mx);
}

getDel2ShadowCnts = function (tag.idx, tags) {
  tag.chunk = tags[tag.idx];

  delNeighs = lapply(tag.chunk, getDel2OnlyNeighbors);  
  del.nrow = sum(unlist(lapply(delNeighs, nrow)))
  # ncol 3 or 4 depending on if 4th col
  del.neigh.mx = matrix(nrow=del.nrow, ncol=4);

  for ( i in 1:length(tag.chunk) ) {
    if ( i==1 ) {
      idx = 1:nrow(delNeighs[[i]]);
    } else {
      idx = (idx[length(idx)]+1):(idx[length(idx)]+nrow(delNeighs[[i]]));
    }
    del.neigh.mx[idx,] = delNeighs[[i]]
  }
  return(del.neigh.mx);
}

getDelNeighbors = function (tag) {
  neighList = lapply(1:(nchar(tag)-1), delOneBase, tag=tag);
  positions = rep(1:(nchar(tag)-1), times=unlist(lapply(neighList, length)))
  tagNeighbors = unlist(neighList);
  cbind(rep(tag, length(tagNeighbors)), positions, tagNeighbors);
}

getDel2OnlyNeighbors = function (tag) {
  neighList = lapply(1:(nchar(tag)-1), delOneBase, tag=tag);
  positions = rep(1:(nchar(tag)-1), times=unlist(lapply(neighList, length)))
  tagNeighbors = unlist(neighList);
  neigh1.mx = cbind(rep(tag, length(tagNeighbors)), positions, tagNeighbors);

  ### need to change the tag column back to the org tag, not the one after
  ### one base of change
  neigh2.list = lapply(tagNeighbors, getDelNeighbors)
  neigh2.mx = neigh2.list[[1]];
  for ( i in 2:length(neigh2.list) ) {
    neigh2.mx = rbind(neigh2.mx, neigh2.list[[i]]);
  }  

  # add original position to col 4
  neigh2.mx = cbind(neigh2.mx, 
    rep(neigh1.mx[,2], times=unlist(lapply(neigh2.list, nrow))))
  
  # which neigh2 is the same as the orginal tag
  idx.tag = which(neigh2.mx[,3] == tag);

  # which neigh2 is the same as neigh1
  idx.neigh1 = which(is.element(neigh2.mx[,3], tagNeighbors));

  # which neigh1 pos is less than neigh2 pos
  idx.pos2lepos1 = which(as.numeric(neigh2.mx[,4]) <= as.numeric(neigh2.mx[,2]))
  idx.rm = c(idx.tag, idx.neigh1, idx.pos2lepos1);
  neigh2.mx = neigh2.mx[-idx.rm,];

  neigh.mx = neigh2.mx;
  neigh.mx[,1] = rep(tag, nrow(neigh.mx))
  return(neigh.mx);
}

insOneBase = function (i, tag) {
  org = tag;
  neighbors = vector(length=4);

  substr(tag, i, i) = "A";
  substr(tag, i+1, nchar(tag)) = substr(org, i, nchar(tag)-1);
  neighbors[1] = tag;
  
  substr(tag, i, i) = "C";
  substr(tag, i+1, nchar(tag)) = substr(org, i, nchar(tag)-1);
  neighbors[2] = tag;
  
  substr(tag, i, i) = "G";
  substr(tag, i+1, nchar(tag)) = substr(org, i, nchar(tag)-1);
  neighbors[3] = tag;
  
  substr(tag, i, i) = "T";
  substr(tag, i+1, nchar(tag)) = substr(org, i, nchar(tag)-1);
  neighbors[4] = tag;

  idx.rm = which(neighbors==org);
  if ( length(idx.rm) > 0 ) {
    neighbors = neighbors[-which(neighbors==org)]
  }
  return(neighbors);
}

delOneBase = function (i, tag) {
  org = tag;
  neighbors = vector(length=4);

  if ( i == 1 ) {
    neighbors = subOneBase(1, tag)
  } else {
    neighbors[1] =
      paste('A', substr(org, 1, i-1), substr(org, i+1, nchar(org)), sep='' )
    neighbors[2] =
      paste('C', substr(org, 1, i-1), substr(org, i+1, nchar(org)), sep='' )
    neighbors[3] =
      paste('G', substr(org, 1, i-1), substr(org, i+1, nchar(org)), sep='' )
    neighbors[4] =
      paste('T', substr(org, 1, i-1), substr(org, i+1, nchar(org)), sep='' )
  }

  idx.rm = which(neighbors==org);
  if ( length(idx.rm) > 0 ) {
    neighbors = neighbors[-which(neighbors==org)]
  }
  return(neighbors);
}

getErrorRates = function(rcnts.file, type="sub") {

  freq.data = read.table(rcnts.file, sep=" ", header=F)
  tags = as.character(freq.data[,1]);
  counts = freq.data[,2];

  idx.N = grep('N', tags);
  idx.A = grep(paste(rep("A", nchar(tags[1])), collapse=''), tags);
  idx.C = grep(paste(rep("C", nchar(tags[1])), collapse=''), tags);
  idx.G = grep(paste(rep("G", nchar(tags[1])), collapse=''), tags);
  idx.T = grep(paste(rep("T", nchar(tags[1])), collapse=''), tags);
  idx.filter = c(idx.N, idx.A, idx.C, idx.G, idx.T);

  if ( length(idx.filter) > 0 ) {
    tags = tags[-idx.filter];
    counts = counts[-idx.filter];
  }

#  if ( length(idx.N) > 0 ) {
#    tags = tags[-idx.N];
#    counts = counts[-idx.N];
#  }

#  tags.entropy = unlist(lapply(tags, function(x) {
#    pi = table(unlist(strsplit(x, ''))) / (nchar(x))
#    return(-sum( pi * log(pi) )) } ));

#  tags.num.bases = unlist(lapply(tags, function(x) {
#    length(unique(unlist(strsplit(x,''))))}))
#  idx.filter = which(tags.num.bases < 4);

#  idx.filter = which(tags.entropy < 0.8 | tags.num.bases < 4); #### threshold???
#  idx.filter = which(tags.entropy < 1 | tags.num.bases < 4); #### threshold???
#  idx.filter = which(tags.entropy < 1.2 | tags.num.bases < 4); #### threshold???

#  if ( length(idx.filter) > 0 ) {
#    tags = tags[-idx.filter];
#    counts = counts[-idx.filter];
#  }

  ts.counts.only2 = getShadow2Cnts.new (1:1000, tags, counts, method=type)
  ts.counts.org = getShadowCnts.new(1:1000, tags, counts, method=type)

  d2neigh.order =
    match(rownames(ts.counts.org[[1]]),
          rownames(ts.counts.only2[[1]]))
  if ( sum(is.na(d2neigh.order)) > 0 ) {
    order.na = which(is.na(d2neigh.order));
    d2neigh.order.noNA = d2neigh.order[-order.na]
    shadow.cnt.d1and2 =
      c(ts.counts.only2[[1]][d2neigh.order.noNA,2] +
        ts.counts.org[[1]][-order.na,2],
        ts.counts.org[[1]][order.na,2])

    shadow.cnt.d1 = rbind(ts.counts.org[[2]][-order.na,],
      ts.counts.org[[2]][order.na,])
    d2.0.mx = matrix(0, length(order.na), ncol(shadow.cnt.d1))
    shadow.cnt.d2 = rbind(ts.counts.only2[[2]][d2neigh.order.noNA,],
      d2.0.mx);

    tag.cnt.new.order = c(ts.counts.org[[1]][-order.na,1],
      ts.counts.org[[1]][order.na,1]);
  } else {
    if ( length(d2neigh.order) > 0 ) {
      d2neigh.order.noNA = d2neigh.order;
      shadow.cnt.d1and2 =
        c(ts.counts.only2[[1]][d2neigh.order.noNA,2] +
          ts.counts.org[[1]][,2])
      shadow.cnt.d1 = ts.counts.org[[2]];
      shadow.cnt.d2 = ts.counts.only2[[2]][d2neigh.order.noNA,];

      tag.cnt.new.order = c(ts.counts.org[[1]][,1])
    } else {
      shadow.cnt.d1and2 = rep(0, length(ts.counts.org[[1]][,1]));
      shadow.cnt.d1 = ts.counts.org[[2]];
      shadow.cnt.d2 = ts.counts.only2[[2]]
      tag.cnt.new.order = c(ts.counts.org[[1]][,1])
    }
  }

  order.s1 = match(names(tag.cnt.new.order), rownames(ts.counts.org[[1]]))
  order.s2 = match(names(tag.cnt.new.order), rownames(ts.counts.only2[[1]]))

  shadow.table.1 = ts.counts.org$shadow.table[order.s1]
  shadow.table.2 = ts.counts.only2$shadow.table[order.s2];

  perReadER = getErrorRates.free.intcpt(tag.cnt.new.order, shadow.cnt.d1and2)
  names(perReadER) = c("error rate", "standard error", "intercept", "slope");

  cycleER = getPositionErrorRates.free.intcpt (tag.cnt.new.order, shadow.cnt.d1, shadow.cnt.d2);
  colnames(cycleER) = c("error rate", "standard error", "intercept", "slope")

  return( list(perReadER=perReadER, cycleER=cycleER, tags=tag.cnt.new.order,
               shadows=shadow.cnt.d1and2, shadows.1 = shadow.cnt.d1,
	       shadows.2=shadow.cnt.d2) );
}

getErrorRates.free.intcpt = function (tag.cnt, shadow.cnt) {
  rlm.out = summary(rlm(shadow.cnt ~ tag.cnt, maxit=100))$coef;
  error.rate = rlm.out[2,1] / (1+rlm.out[2,1]);
  er.stderr = 1/((1+rlm.out[2,1])^2) * rlm.out[2,2];
  intercept = rlm.out[1,1];
  slope = rlm.out[2,1];
  return(c(error.rate, er.stderr, intercept, slope));
}

getPositionErrorRates.free.intcpt = function (tag.cnt, shadow.cnt.d1, shadow.cnt.d2) {
  shadow.d1.sum = apply(shadow.cnt.d1, 1, sum);

  cycleER = matrix(NA, nrow=ncol(shadow.cnt.d1), ncol=4);
  for ( k in 1:ncol(shadow.cnt.d1) ) {
    shadow.d1.notK = shadow.d1.sum - shadow.cnt.d1[,k];
    shadow.total.k = shadow.cnt.d1[,k] + shadow.cnt.d2[,k];
    tag.adjust = tag.cnt + shadow.d1.notK;

    rlm.out = summary(rlm(shadow.total.k ~ tag.adjust, maxit=100))$coef;
    error.rate = rlm.out[2,1] / (1+rlm.out[2,1]);
    er.stderr = 1/((1+rlm.out[2,1])^2) * rlm.out[2,2];
    intercept = rlm.out[1,1];
    slope = rlm.out[2,1];
    cycleER[k,] = c(error.rate, er.stderr, intercept, slope);
  }
  cycleER[1,] = NA;
  cycleER[nrow(cycleER),] = NA;
  return(cycleER);
}

# gfile should be in fasta format
sampleFromGenome = function(gfile, sample.n=30000000, rlength=35) {
  seqs = readLines(gfile)
  seq = paste(seqs[-1], sep='', collapse='');
  starts = sample(1:(nchar(seq)-rlength+1), sample.n, rep=T);
  reads = lapply(starts, function(x) {
    read = substr(seq, x, (x+rlength-1));
  })
  reads.char = unlist(reads);
  read.cnts = table(reads.char);
  read.cnts.sort = sort(read.cnts, decreasing=T);
  srfile = paste(gfile, 'rcnts', sep='.')
  write.table(read.cnts.sort, file=srfile, quote=F, col.names=F);
}

plot.SR = function (ER) {
  plot(ER$tags, ER$shadows, xlab="tag count", ylab="shadow count", pch=20,
       cex=0.5);
  abline(ER$perReadER[3], ER$perReadER[4])
}
